=head1 NAME

Slapd::Backend::Cyrus::User - mail users' LDAP slapd backend class

=head1 DESCRIPTION

Synchronization mail users' folders between LDAP and Cyrus.

=cut

package Slapd::Backend::Cyrus::User;

use utf8;
use strict;
use warnings;
use base 'Slapd::Backend::Cyrus::Base';

use BerkeleyDB;
use Net::LDAP::Constant qw/
	LDAP_SUCCESS
	LDAP_NO_SUCH_OBJECT
	LDAP_UNWILLING_TO_PERFORM
	LDAP_ALREADY_EXISTS
/;

use constant stored_entries => qw/user/;
use constant stored_attributes => qw/uid mailLocalAddress/;
use constant key_attribute => qw/uid/;

=head1 PROTECTED METHODS

=over

=item B<_search>( filter, entries )

Purpose:
  * Search

Arguments:
  * filter : Hashref - search filter
  * entries : Arrayref of Net::LDAP::Entry - search in these entries

Returns:
  * Scalar - result code
  * Array  - Net::LDAP::Entry entries

=cut

sub _search
{
	my $self = shift;
	my %in = @_;

	my %filter = $in{filter} ? %{$in{filter}} : ();
	my @entries = @{$in{entries}};

	my @match_entries;
	foreach my $entry ( @entries )
	{
		my @mail = $self->_user_mail( $entry );
		if( !exists $filter{mailLocalAddress} || $filter{mailLocalAddress} ~~ \@mail )
		{
			my $full_entry = $entry->clone();
			$full_entry->replace( mailLocalAddress => \@mail );
			push @match_entries, $full_entry;
		}
	}

	return ( LDAP_SUCCESS, @match_entries );
}

=item B<_modify>( entry, actions )

Purpose:
  * Modify

Arguments:
  * entry : Net::LDAP::Entry - compared entry
  * actions : ArrayRef - list of actions

Returns:
  * Scalar - result code

=cut

sub _modify
{
	my $self = shift;
	my %in = @_;

	my $entry = $in{entry};
	my @actions = @{$in{actions}};

	foreach my $action ( @actions )
	{
		if( $self->_is_user( $entry ) && $action->{attr} eq 'mailLocalAddress' )
		{
			if( $action->{action} ~~ ['ADD', 'REPLACE'] )
			{
				$self->_apply_mails( $entry, $action->{values} );
			}
			my %mail = map { $_ => 1 } $self->_user_mail( $entry );
			if( $action->{action} eq 'ADD' )
			{
				$mail{$_} = 1 foreach @{$action->{values}};
			}
			elsif( $action->{action} eq 'REPLACE' )
			{
				%mail = map { $_ => 1 } @{$action->{values}};
			}
			elsif( $action->{action} eq 'DELETE' )
			{
				delete @mail{ @{$action->{values}} };
			}
			my $status = $self->usermail_db->db_put( $entry->get_value('uid'), join ' ', sort keys %mail );
			warn $status unless $status == 0;
		}
		else
		{
			return LDAP_UNWILLING_TO_PERFORM;
		}
	}

	return LDAP_SUCCESS;
}

=item B<_add>( entry )

Purpose:
  * Add

Arguments:
  * entry : Net::LDAP::Entry - added entry

Returns:
  * Scalar - result code

=cut

sub _add
{
	my $self = shift;
	my %in = @_;

	my $entry = $in{entry};

	if( $self->_is_user( $entry ) )
	{
		my $status = $self->usermail_db->db_put( $entry->get_value('uid'), join ' ', $entry->get_value('mailLocalAddress') );
		warn $status unless $status == 0;
		return $self->_apply_mails( $entry );
	}
	else
	{
		return LDAP_UNWILLING_TO_PERFORM;
	}

	return LDAP_SUCCESS;
}

=item B<_delete>( entry )

Purpose:
  * Delete

Arguments:
  * entry : Net::LDAP::Entry - entry

Returns:
  * None

=cut

sub _delete
{
	my $self = shift;
	my %in = @_;

	my $entry = $in{entry};

	if( $self->_is_user( $entry ) )
	{
		my $status = $self->usermail_db->db_del( $entry->get_value('uid') );
		warn $status unless $status ~~ [0, DB_NOTFOUND];
	}
	else
	{
		return LDAP_UNWILLING_TO_PERFORM;
	}

	return LDAP_SUCCESS;
}

=item B<_config>( $name, $value )

Purpose:
  * Config

Arguments:
  * Scalar - name
  * Scalar - value

Returns:
  * Scalar - result code

=cut

sub _config
{
	my $self = shift;
	my( $name, $value ) = @_;
	if( $name eq 'cyrusUser' )
	{
		$self->{user_filter} = $self->parse_filter( $value );
		return LDAP_SUCCESS;
	}
	else
	{
		return $self->SUPER::_config(@_);
	}
}

=back

=head1 PRIVATE METHODS

=over

=item B<_is_user>( $entry )

Purpose:
  * Check if C<$entry> is user

Arguments:
  * Net::LDAP::Entry object - entry

Returns:
  * Scalar - boolean

=cut

sub _is_user
{
	my( $self, $entry ) = @_;
	return $self->check_filter( $entry, $self->{user_filter} );
}

=item B<_domain_exists>( $domain )

Purpose:
  * Check if domain exists

Arguments:
  * Scalar - domain

Returns:
  * Scalar - boolean

=cut

sub _domain_exists
{
	my( $self, $domain ) = @_;
	my $exists;
	my $status = $self->domain_db->db_get( $domain, $exists );
	return $status == 0 && $exists;
}

=item B<_user_mail>( $entry )

Purpose:
  * Returns mail list for C<$entry>

Arguments:
  * Net::LDAP::Entry object - entry

Returns:
  * List - mail

=cut

sub _user_mail
{
	my( $self, $entry ) = @_;
	my $mails = '';
	my $status = $self->usermail_db->db_get( $entry->get_value('uid'), $mails );
	warn $status unless $status ~~ [0, DB_NOTFOUND];
	my @mail;
	foreach my $mail ( split /\s+/, $mails )
	{
		my $domain = $mail;
		$domain =~ s/^.*@//;
		push @mail, $mail if $self->cyradm->listaclmailbox( "user/" . $entry->get_value('uid') . "\@$domain" );
	}
	return @mail;
}

=item B<_apply_mails>( $entry, $mails )

Purpose:
  * Apply mail list for C<$entry>

Arguments:
  * Net::LDAP::Entry object - entry
  * Arrayref - mail list (optional)

Returns:
  * Scalar - ldap error

=cut

sub _apply_mails
{
	my( $self, $entry, $mails ) = @_;
	my %domains;
	foreach my $mail ( $mails ? @$mails : $entry->get_value('mailLocalAddress') )
	{
		my $domain = $mail;
		$domain =~ s/^.*@//;
		$domains{$domain} = 1;
	}
	foreach my $domain ( keys %domains )
	{
		if( $self->_domain_exists( $domain ) )
		{
			my $mailbox = "user/" . $entry->get_value('uid') . "\@$domain";
			unless( $self->cyradm->listmailbox( $mailbox ) )
			{
				unless( $self->cyradm->createmailbox( $mailbox ) )
				{
					my $error = $self->_cyradm_error();
					return $error if $error != LDAP_ALREADY_EXISTS;
				}
			}
		}
	}
	return LDAP_SUCCESS;
}

=back

=cut

1;

=head1 SEE ALSO

L<Slapd::Backend::Cyrus::Base>, L<Slapd::Backend::Cyrus>, L<Slapd::Backend::Proxy>, L<slapd-perl>

=head1 AUTHOR

Aleksey Mashanov <aleksey.mashanov@gmail.com>

=head1 LICENSE

This library is free software, you can redistribute it and/or modify it under
the same terms as Perl itself.

=cut
